@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
-from django.db import models
+from django.db import models, transaction
 from django.db.models import Q
 from django.db.models.query import QuerySet
+from django.db.utils import IntegrityError
 from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
 from django.conf import settings
 from vkontakte_api import fields
@@ -14,6 +15,7 @@ import logging
 
 log = logging.getLogger('vkontakte_groups_migration')
 
+FETCH_ONLY_EXPIRED_USERS = getattr(settings, 'VKONTAKTE_GROUPS_MIGRATION_FETCH_ONLY_EXPIRED_USERS', True)
 
 def ModelQuerySetManager(ManagerBase=models.Manager):
     '''
@@ -53,6 +55,29 @@ class GroupMigrationQueryset(object):
 
 class GroupMigrationManager(models.Manager, GroupMigrationQueryset):
 
+    def fix_wrong_memberships_count(self, group):
+
+        migr = group.migrations.latest('id')
+
+        while True:
+            if migr.check_memberships_count():
+                break
+            else:
+                migr = migr.prev
+
+        migr.clear_future_users_memberships()
+
+        while True:
+            try:
+                migr.save_final()
+                migr.check_memberships_count()
+                migr = migr.next
+            except AttributeError:
+                break
+
+        migr = group.migrations.latest('id')
+        print 'Group %s: %s == %s' % (group, GroupMembership.objects.get_user_ids(group).count(), migr.members_count)
+
     @opt_generator
     def update_for_group(self, group, offset=0):
         '''
@@ -104,7 +129,7 @@ class GroupMigration(models.Model):
     group = models.ForeignKey(Group, verbose_name=u'Группа', related_name='migrations')
     time = models.DateTimeField(u'Дата и время', null=True, db_index=True)
 
-    hidden = models.BooleanField(u'Скрыть', db_index=True)
+    hidden = models.BooleanField(u'Скрыть', default=False, db_index=True)
 
     offset = models.PositiveIntegerField(default=0)
 
@@ -140,39 +165,39 @@ class GroupMigration(models.Model):
         self.members_has_avatar_left_ids = []
 
     @property
-    @memoize
     def next(self):
-        try:
-            return self.group.migrations.visible.filter(time__gt=self.time).order_by('time')[0]
-        except IndexError:
-            return None
+        return self.get_next()
 
     @property
-    @memoize
     def prev(self):
+        return self.get_prev()
+
+    def get_next(self, step=0):
         try:
-            return self.group.migrations.visible.filter(time__lt=self.time).order_by('-time')[0]
+            return self.group.migrations.visible.filter(time__gt=self.time).order_by('time')[step]
+        except IndexError:
+            return None
+
+    def get_prev(self, step=0):
+        try:
+            return self.group.migrations.visible.filter(time__lt=self.time).order_by('-time')[step]
         except IndexError:
             return None
 
     @property
-    @memoize
+#    @memoize
     def user_ids(self):
-        return GroupMembership.objects.filter(group=self.group) \
-            .filter(Q(time_entered=None, time_left=None) | \
-                    Q(time_entered=None, time_left__gt=self.time) | \
-                    Q(time_entered__lte=self.time, time_left=None) | \
-                    Q(time_entered__lte=self.time, time_left__gt=self.time)).values_list('user_id', flat=True)
+        return GroupMembership.objects.get_user_ids(self.group, self.time)
 
     @property
-    @memoize
+#    @memoize
     def entered_user_ids(self):
-        return GroupMembership.objects.filter(group=self.group).filter(time_entered=self.time).values_list('user_id', flat=True)
+        return GroupMembership.objects.get_entered_user_ids(self.group, self.time)
 
     @property
-    @memoize
+#    @memoize
     def left_user_ids(self):
-        return GroupMembership.objects.filter(group=self.group).filter(time_left=self.time).values_list('user_id', flat=True)
+        return GroupMembership.objects.get_left_user_ids(self.group, self.time)
 
     def delete(self, *args, **kwargs):
         '''
@@ -189,6 +214,12 @@ class GroupMigration(models.Model):
         self.save()
 
     def fix_memberships(self):
+        '''
+        Fixes memberships timeline after hiding current migration
+        TODO: now works only for hiding, teach method to work for unhiding cases
+        '''
+        if not self.hidden:
+            return
 
         if self.next:
             # delete memberships stopped in current and started from the next migration, we need to join them
@@ -214,6 +245,18 @@ class GroupMigration(models.Model):
         # update rest of left users -> they are not left
         GroupMembership.objects.filter(group=self.group, user_id__in=self.members_left_ids, time_left=self.time).update(time_left=None)
 
+    def check_memberships_count(self):
+        '''
+        Compares ammount of memberships and members_count value and returns True if they are equal or False otherwise
+        '''
+        memberships_count = self.user_ids.count()
+        if memberships_count == self.members_count:
+            log.info('%s - %s: %s == %s' % (self.group, self.time, memberships_count, self.members_count))
+            return True
+        else:
+            log.info('%s - %s: %s != %s' % (self.group, self.time, memberships_count, self.members_count))
+            return False
+
     def update_next(self):
         next_stat = self.next
         if next_stat:
@@ -221,9 +264,11 @@ class GroupMigration(models.Model):
             next_stat.save()
 
     def save(self, *args, **kwargs):
-        update_next = False
-        if self.id and self.hidden != self.__class__.objects.light.get(id=self.id).hidden:
+        try:
+            assert self.hidden != self.__class__.objects.light.get(pk=self.pk).hidden
             update_next = True
+        except:
+            update_next = False
 
         super(GroupMigration, self).save(*args, **kwargs)
 
@@ -239,35 +284,47 @@ class GroupMigration(models.Model):
         self.offset = 0
         self.clean_members()
         self.update()
-        self.update_users_memberships()
         self.save()
+        # call only after saving migrations, because in case of fault we need to have right migrations as source for memberships
+        self.update_users_memberships()
 
-    def compare_with_previous(self):
-        if self.hidden or not self.prev:
+    def compare_with_siblings(self):
+        if self.hidden or not self.prev or self.members_count < 10000:
             return
 
         delta = self.time - self.prev.time
 
-        if delta > timedelta(7):
+        if delta > timedelta(2):
             return
 
-        division = float(self.members_count) / self.prev.members_count
-        if 0.9 < division < 1.1:
-            return
-        else:
-            log.warning("Suspicious migration found. Previous value is %d, current value is %d, time delta %s. Group %s, migration ID %d" % (self.prev.members_count, self.members_count, delta, self.group, self.id))
-            self.hide()
+        def check(count1, count2):
+            if not count1 or not count2:
+                return True
+            division = float(count1) / count2
+            value = float('%f' % abs(1 - division)) # otherways it will be 0.09999999999999998
+            return value >= 0.1
+
+        if check(self.members_count, self.prev.members_count):
+            if not self.next:
+                log.warning("Suspicious migration found. Current value is %d, previous value is %d, time delta %s. Group %s, migration ID %d" % (self.prev.members_count, self.members_count, delta, self.group, self.id))
+                self.hide()
+            else:
+                delta_next = self.next.time - self.time
+                if check(self.members_count, self.next.members_count):
+                    log.warning("Suspicious migration found. Current value is %d, previous value is %d, time delta %s, next value is %d, time delta %s, Group %s, migration ID %d" % (self.members_count, self.prev.members_count, delta, self.next.members_count, delta_next, self.group, self.id))
+                    self.hide()
+                elif check(self.next.members_count, self.prev.members_count):
+                    log.warning("Suspicious previous migration found. Current value is %d, previous value is %d, time delta %s, next value is %d, time delta %s, Group %s, migration ID %d" % (self.members_count, self.prev.members_count, delta, self.next.members_count, delta_next, self.group, self.id))
+                    self.prev.hide()
 
     def compare_entered_left(self):
-        if self.hidden or not self.prev:
+        if self.hidden or not self.prev or self.members_left_count <= 5000 or self.members_entered_count == 0:
             return
 
         delta = self.time - self.prev.time
 
         division = float(self.members_entered_count) / self.members_left_count
-        if 1/100 < division < 100:
-            return
-        else:
+        if division < 0.05:
             log.warning("Suspicious migration found. Ammounts members entered is %d, left is %d, time delta %s. Group %s, migration ID %d" % (self.members_entered_count, self.members_left_count, delta, self.group, self.id))
             self.hide()
 
@@ -276,14 +333,13 @@ class GroupMigration(models.Model):
             assert not self.hidden
             assert 'vkontakte_groups_statistic' in settings.INSTALLED_APPS
             members_count = self.group.statistics.get(date=self.time.date(), period=1).members
+            assert members_count
         except (ObjectDoesNotExist, AssertionError):
             return
 
-        division = float(self.members_count) / members_count
-        if 0.98 < division < 1.02:
-            return
-        else:
-            log.warning("Suspicious migration found. Statistic value is %d, API value is %d. Group %s, migration ID %d" % (members_count, self.members_count, self.group, self.id))
+        delta = abs(members_count - self.members_count)
+        if delta >= 1000:
+            log.warning("Suspicious migration found. API value is %d, statistic value is %d, delta is %s. Group %s, migration ID %d" % (members_count, self.members_count, delta, self.group, self.id))
             self.hide()
 
     def clean_members(self):
@@ -303,14 +359,17 @@ class GroupMigration(models.Model):
         if self.prev and self.group:
             self.members_left_ids = list(set(prev_stat.members_ids).difference(set(self.members_ids)))
             self.members_entered_ids = list(set(self.members_ids).difference(set(prev_stat.members_ids)))
+        else:
+            self.members_left_ids = []
+            self.members_entered_ids = []
 
     def update_deactivated(self):
         self.members_deactivated_entered_ids = list(User.objects.deactivated().filter(remote_id__in=self.members_entered_ids).values_list('remote_id', flat=True))
         self.members_deactivated_left_ids = list(User.objects.deactivated().filter(remote_id__in=self.members_left_ids).values_list('remote_id', flat=True))
 
     def update_with_avatar(self):
-        self.members_has_avatar_entered_ids = list(User.objects.has_avatars().filter(remote_id__in=self.members_entered_ids).values_list('remote_id', flat=True))
-        self.members_has_avatar_left_ids = list(User.objects.has_avatars().filter(remote_id__in=self.members_left_ids).values_list('remote_id', flat=True))
+        self.members_has_avatar_entered_ids = list(User.objects.with_avatar().filter(remote_id__in=self.members_entered_ids).values_list('remote_id', flat=True))
+        self.members_has_avatar_left_ids = list(User.objects.with_avatar().filter(remote_id__in=self.members_left_ids).values_list('remote_id', flat=True))
 
     def update_counters(self):
         for field_name in ['members','members_entered','members_left','members_deactivated_entered','members_deactivated_left','members_has_avatar_entered','members_has_avatar_left']:
@@ -321,26 +380,38 @@ class GroupMigration(models.Model):
         Fetch all users of group, make new m2m relations, remove old m2m relations
         '''
         log.debug('Fetching users for the group "%s"' % self.group)
-        User.remote.fetch(ids=self.user_ids, only_expired=True)
+        User.remote.fetch(ids=self.user_ids, only_expired=FETCH_ONLY_EXPIRED_USERS)
 
-        # process entered nad left users of the group
-        # here is possible using relative self.members_*_ids, but it's better absolute values, calculated by self.group.users
+        # process entered and left users of the group
+        # here is possible using relative self.members_*_ids, but it's better absolute values, calculated from self.group.users
         ids_current = self.group.users.values_list('remote_id', flat=True)
         ids_left = set(ids_current).difference(set(self.members_ids))
         ids_entered = set(self.members_ids).difference(set(ids_current))
 
         log.debug('Adding %d new users to the group "%s"' % (len(ids_entered), self.group))
         ids = User.objects.filter(remote_id__in=ids_entered).values_list('pk', flat=True)
-        self.group.users.through.objects.bulk_create([self.group.users.through(group_id=self.group.pk, user_id=id) for id in ids])
+        self.group.users.through.objects.bulk_create([self.group.users.through(group=self.group, user_id=id) for id in ids])
 
         log.info('Removing %d left users from the group "%s"' % (len(ids_left), self.group))
         ids = User.objects.filter(remote_id__in=ids_left).values_list('pk', flat=True)
-        self.group.users.through.objects.filter(group_id=self.group.pk, user_id__in=ids).delete()
+        self.group.users.through.objects.filter(group=self.group, user_id__in=ids).delete()
 
         signals.group_users_updated.send(sender=Group, instance=self.group)
         log.info('Updating m2m relations of users for group "%s" successfuly finished' % self.group)
         return True
 
+    def clear_future_users_memberships(self):
+        '''
+        Method:
+         * removes all entered and left users after and during current migration
+         * removes all entered and not left after and during current migration
+         * makes all left users after and during current migration not left
+        '''
+        GroupMembership.objects.filter(group=self.group, time_left__gte=self.time, time_entered__gte=self.time).delete()
+        GroupMembership.objects.filter(group=self.group, time_entered__gte=self.time, time_left=None).delete()
+        GroupMembership.objects.filter(group=self.group, time_left__gte=self.time).update(time_left=None)
+
+    @transaction.commit_on_success
     def update_users_memberships(self):
         '''
         Fetch all users of group, make new m2m relations, remove old m2m relations
@@ -353,18 +424,19 @@ class GroupMigration(models.Model):
             GroupMembership.objects.bulk_create([GroupMembership(group=self.group, user_id=user_id) for user_id in self.members_ids])
         else:
             # ensure current number of memberships equal to members in previous migration
-            memberships_count = GroupMembership.objects.filter(group=self.group, time_left=None).count()
-            if self.prev.members_count != memberships_count:
+            memberships_count = GroupMembership.objects.get_user_ids(self.group).count()
+            members_count = self.prev.members_count
+            if members_count != memberships_count:
                 # something wrong with previous migration
-                raise Exception("Number of current memberships %d is not equal to members count %d of previous migration, group %s at %s" % (memberships_count, self.prev.members_count, self.group, self.time))
+                raise Exception("Number of current memberships %d is not equal to members count %d of previous migration, group %s at %s" % (memberships_count, members_count, self.group, self.time))
 
             # ensure entered users not in memberships now
-            error_ids_count = GroupMembership.objects.filter(group=self.group, time_left=None, user_id__in=self.members_entered_ids).count()
+            error_ids_count = GroupMembership.objects.get_user_ids(self.group).filter(user_id__in=self.members_entered_ids).count()
             if error_ids_count != 0:
                 raise Exception("Found %d just entered users, that still not left from the group %s at %s" % (error_ids_count, self.group, self.time))
 
             # ensure left users in memberships now
-            left_ids_count = GroupMembership.objects.filter(group=self.group, time_left=None, user_id__in=self.members_left_ids).count()
+            left_ids_count = GroupMembership.objects.get_user_ids(self.group).filter(user_id__in=self.members_left_ids).count()
             if left_ids_count != self.members_left_count:
                 raise Exception("Not all left users found %d != %d between active in group %s at %s" % (left_ids_count, self.members_left_count, self.group, self.time))
 
@@ -378,6 +450,26 @@ class GroupMigration(models.Model):
 
 class GroupMembershipManager(models.Manager):
 
+    def get_user_ids(self, group, time=None):
+        if time is None:
+            qs = self.filter(group=group, time_left=None)
+        else:
+            qs = self.filter(group=group) \
+                .filter(Q(time_entered=None,        time_left=None) | \
+                        Q(time_entered=None,        time_left__gt=time) | \
+                        Q(time_entered__lte=time,   time_left=None) | \
+                        Q(time_entered__lte=time,   time_left__gt=time))
+
+        return qs.order_by('user_id').distinct('user_id').values_list('user_id', flat=True)
+
+    def get_entered_user_ids(self, group, time):
+        return self.filter(group=group).filter(time_entered=time) \
+            .order_by('user_id').distinct('user_id').values_list('user_id', flat=True)
+
+    def get_left_user_ids(self, group, time):
+        return self.filter(group=group).filter(time_left=time) \
+            .order_by('user_id').distinct('user_id').values_list('user_id', flat=True)
+
     def get_user_ids_of_period(self, group, date_from, date_to, field=None):
 
         if field is None:
@@ -392,14 +484,11 @@ class GroupMembershipManager(models.Manager):
 
         return self.filter(group=group, **kwargs).order_by('user_id').distinct('user_id').values_list('user_id', flat=True)
 
-    def get_active_users_ids_of_date(self, group, date):
-        return self.filter(group=group, **kwargs).order_by('user_id').distinct('user_id').values_list('user_id', flat=True)
-
 class GroupMembership(models.Model):
     class Meta:
         verbose_name = u'Членство пользователя группы Вконтакте'
         verbose_name_plural = u'Членства пользователей групп Вконтакте'
-        unique_together = (('group','user_id','time_entered'), ('group','user_id','time_left'),)
+#        unique_together = (('group','user_id','time_entered'), ('group','user_id','time_left'),)
         ordering = ('group', 'user_id', 'id')
 
     group = models.ForeignKey(Group, verbose_name=u'Группа', related_name='memberships')
@@ -410,4 +499,18 @@ class GroupMembership(models.Model):
 
     objects = GroupMembershipManager()
 
+    def save(self, *args, **kwargs):
+        # TODO: perhaps useless checkings, since all GroupMemberships are created by bulk_create..
+
+        if self.time_entered and self.time_left and self.time_entered > self.time_left:
+            raise IntegrityError("GroupMembership couldn't have time_entered (%s) > time_left (%s), group %s, user remote ID %s" % (self.time_entered, self.time_left, self.group, self.user_id))
+
+        # check additionally null values of time_entered and time_left,
+        # because for postgres null values are acceptable in unique constraint
+        if not self.time_entered and self.__class__.objects.filter(group=self.group, user_id=self.user_id, time_entered=None).count() != 0 \
+            or not self.time_left and self.__class__.objects.filter(group=self.group, user_id=self.user_id, time_left=None).count() != 0:
+                raise IntegrityError("columns group_id, user_id, time_entered are not unique")
+
+        return super(GroupMembership, self).save(*args, **kwargs)
+
 import signals
