@@ -1,16 +1,20 @@
 # -*- coding: utf-8 -*-
 from django.test import TestCase
+from django.test.testcases import TransactionTestCase
 from django.conf import settings
+from django.db.utils import IntegrityError
 from models import GroupMigration, User
 from vkontakte_users.factories import UserFactory
 from vkontakte_groups.factories import GroupFactory
-from factories import GroupMigrationFactory, GroupMembership
+from factories import GroupMigrationFactory, GroupMembershipFactory, GroupMembership
 from datetime import datetime, timedelta
 
 GROUP_ID = 30221121
 
 class VkontakteGroupsMigrationTest(TestCase):
 
+    maxDiff = None
+
     def test_user_memberships(self):
 
         migration1 = GroupMigrationFactory(time=datetime.now() - timedelta(3), members_ids=range(30, 100))
@@ -184,13 +188,13 @@ class VkontakteGroupsMigrationTest(TestCase):
         migration.update()
         migration.save()
 
-        stat = GroupStatFactory(group=migration.group, members=9800, date=migration.time.date())
+        stat = GroupStatFactory(group=migration.group, members=9000, date=migration.time.date())
 
         self.assertEqual(migration.hidden, False)
         migration.compare_with_statistic()
         self.assertEqual(migration.hidden, True)
 
-        stat.members = 9810
+        stat.members = 9001
         stat.save()
         migration.hidden = False
 
@@ -204,7 +208,7 @@ class VkontakteGroupsMigrationTest(TestCase):
         migration1.update()
         migration1.save()
 
-        migration2 = GroupMigrationFactory(group=migration1.group, time=datetime.now() - timedelta(1), members_ids=range(999, 1010))
+        migration2 = GroupMigrationFactory(group=migration1.group, time=datetime.now() - timedelta(1), members_ids=range(99990, 100010))
         migration2.update()
         migration2.save()
 
@@ -212,28 +216,50 @@ class VkontakteGroupsMigrationTest(TestCase):
         migration2.compare_entered_left()
         self.assertEqual(migration2.hidden, True)
 
-    def test_comparing_with_previous(self):
+    def test_comparing_with_siblings(self):
 
-        migration1 = GroupMigrationFactory(time=datetime.now() - timedelta(2), members_ids=range(0, 10000))
+        migration1 = GroupMigrationFactory(time=datetime.now() - timedelta(2), members_ids=range(0, 100000))
         migration1.update()
         migration1.save()
 
-        migration2 = GroupMigrationFactory(group=migration1.group, time=datetime.now() - timedelta(1), members_ids=range(0, 9000))
+        migration2 = GroupMigrationFactory(group=migration1.group, time=datetime.now() - timedelta(1), members_ids=range(0, 90000))
         migration2.update()
         migration2.save()
 
         self.assertEqual(migration2.hidden, False)
-        migration2.compare_with_previous()
+        migration2.compare_with_siblings()
         self.assertEqual(migration2.hidden, True)
 
-        migration3 = GroupMigrationFactory(group=migration1.group, members_ids=range(0, 9001))
+        migration3 = GroupMigrationFactory(group=migration1.group, members_ids=range(0, 90010))
         migration3.update()
         migration3.save()
 
         self.assertEqual(migration3.hidden, False)
-        migration3.compare_with_previous()
+        migration3.compare_with_siblings()
         self.assertEqual(migration3.hidden, False)
 
+        # comparing with previous and next
+        GroupMigration.objects.all().delete()
+        migration1 = GroupMigrationFactory(time=datetime.now() - timedelta(2), members_ids=range(0, 90000))
+        migration1.update()
+        migration1.save()
+
+        migration2 = GroupMigrationFactory(group=migration1.group, time=datetime.now() - timedelta(1), members_ids=range(0, 100000))
+        migration2.update()
+        migration2.save()
+
+        migration3 = GroupMigrationFactory(group=migration1.group, members_ids=range(0, 100010))
+        migration3.update()
+        migration3.save()
+
+        self.assertEqual(migration1.hidden, False)
+        self.assertEqual(migration2.hidden, False)
+        self.assertEqual(migration3.hidden, False)
+        migration2.compare_with_siblings()
+        self.assertEqual(GroupMigration.objects.get(id=migration1.id).hidden, True)
+        self.assertEqual(GroupMigration.objects.get(id=migration2.id).hidden, False)
+        self.assertEqual(GroupMigration.objects.get(id=migration3.id).hidden, False)
+
     def test_m2m_relations(self):
 
         [UserFactory(remote_id=i, fetched=datetime.now()) for i in range(0, 20)]
@@ -300,4 +326,35 @@ class VkontakteGroupsMigrationTest(TestCase):
 
         # difference between stat5 and stat1
         self.assertItemsEqual(stat5.members_entered_ids, [7])
-        self.assertItemsEqual(stat5.members_left_ids, [4])
\ No newline at end of file
+        self.assertItemsEqual(stat5.members_left_ids, [4])
+
+class VkontakteGroupsMembershipsTest(TransactionTestCase):
+
+    def test_memberships_restrictions1(self):
+        kwargs = dict(group=GroupFactory(), user_id=1, time_entered=datetime.now(), time_left=datetime.now()-timedelta(1))
+        with self.assertRaises(IntegrityError):
+            GroupMembershipFactory(**kwargs)
+
+    def test_memberships_restrictions2(self):
+        kwargs = dict(group=GroupFactory(), user_id=1, time_entered=datetime.now())
+        GroupMembershipFactory(**kwargs)
+        with self.assertRaises(IntegrityError):
+            GroupMembershipFactory(**kwargs)
+
+    def test_memberships_restrictions3(self):
+        kwargs = dict(group=GroupFactory(), user_id=1, time_left=datetime.now())
+        GroupMembershipFactory(**kwargs)
+        with self.assertRaises(IntegrityError):
+            GroupMembershipFactory(**kwargs)
+
+    def test_memberships_restrictions4(self):
+        kwargs = dict(group=GroupFactory(), user_id=1, time_entered=None)
+        GroupMembershipFactory(**kwargs)
+        with self.assertRaises(IntegrityError):
+            GroupMembershipFactory(**kwargs)
+
+    def test_memberships_restrictions5(self):
+        kwargs = dict(group=GroupFactory(), user_id=1, time_left=None)
+        GroupMembershipFactory(**kwargs)
+        with self.assertRaises(IntegrityError):
+            GroupMembershipFactory(**kwargs)
\ No newline at end of file
